#include <sys/poll.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <errno.h>

#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#include <linux/dvb/dmx.h>
#include <linux/dvb/frontend.h>

#include "getstream.h"
#include "dvbrc.h"

/*
 * Dump FrontEnd Status byte as clear text returned
 * by GET_STATUS or GET_EVENT ioctl
 *
 */
char *fe_decode_status(int status) {
	static char	str[256];

	str[0]=0x0;

	if (status & FE_HAS_SIGNAL) 
		strcat(str, "HAS_SIGNAL ");
	if (status & FE_HAS_CARRIER)
		strcat(str, "HAS_CARRIER ");
	if (status & FE_HAS_VITERBI) 
		strcat(str, "HAS_VITERBI ");
	if (status & FE_HAS_SYNC) 
		strcat(str, "HAS_SYNC ");
	if (status & FE_HAS_LOCK) 
		strcat(str, "HAS_LOCK ");
	if (status & FE_TIMEDOUT) 
		strcat(str, "TIMEDOUT ");
	if (status & FE_REINIT) 
		strcat(str, "REINIT ");

	return str;
}

/*
 * Wait for the frontend to deliver an event
 * Return to callee if either TIMEDOUT or HAS_LOCK
 *
 */
int fe_get_event(int fd) {
	struct pollfd pfd;
	struct dvb_frontend_event event;
	int	res;

	pfd.fd = fd;
	pfd.events = POLLIN;

	do {
		if (!poll(&pfd, 1, 10000))
			continue;

		if (!(pfd.revents & POLLIN))
			continue;

		res=ioctl(fd, FE_GET_EVENT, &event);

		/*
		 * EOVERFLOW happens if we didnt poll for events
		 * lately and the event queue (8) overflowed inbetween.
		 *
		 */
		if (res < 0 && errno != EOVERFLOW) {
			perror("FE_GET_EVENT failed");
			exit(-1);
		}

		if (res >= 0 && event.status)
			dprintf("Status: %02x %s\n", event.status, fe_decode_status(event.status));

	} while (((event.status & FE_HAS_LOCK) == 0) && ((event.status & FE_TIMEDOUT) == 0));

	return(event.status);
}

int fe_tune(struct adapter_s *adapter) {
	struct dvb_frontend_parameters	feparams;
	struct dvbrc_transponder_s	*t=adapter->transponder;
	struct dvbrc_lnb_s		*lnb;
	char				fename[128];
	int				voltage, tone, status;

	memset(&feparams, 0, sizeof(struct dvb_frontend_parameters));

	sprintf(fename, "/dev/dvb/adapter%d/frontend0", adapter->no);

	adapter->fefd=open(fename, O_RDWR);

	if (!adapter->fefd) {
		perror("Error opening dvb frontend");
		exit(-1);
	}

	voltage=(t->pol == 'H') ? SEC_VOLTAGE_18 : SEC_VOLTAGE_13;

	/* TODO - This is all DVB-S/QPSK - DVB-C/DVB-T ? */
	if (ioctl(adapter->fefd, FE_SET_VOLTAGE, voltage) < 0) {
		fprintf(stderr, "ioctl FE_SET_VOLTAGE failed\n");
		exit(-1);
	}

	lnb=t->satellite->lnb;

	if (t->freq > 2200000) {
		if (t->freq < lnb->slof) {
			feparams.frequency=(t->freq-lnb->lof1);
			tone = SEC_TONE_OFF;
		} else {
			feparams.frequency=(t->freq-lnb->lof2);
			tone = SEC_TONE_ON;
		}
	} else {
		feparams.frequency=t->freq;
	}

	feparams.inversion=INVERSION_AUTO;
	feparams.u.qpsk.symbol_rate=t->srate;
	feparams.u.qpsk.fec_inner=FEC_AUTO;

	if (ioctl(adapter->fefd, FE_SET_TONE, tone) < 0) {
		perror("ERROR setting tone\n");
	}

#ifdef DEBUG
	printf("FE_SET_FRONTEND:\n");
	printf("\ttone = %d\n", tone);
	printf("\tt->freq = %lu\n", t->freq);
	printf("\tlnb->lof1 = %lu\n", lnb->lof1);
	printf("\tlnb->lof2 = %lu\n", lnb->lof2);
	printf("\tlnb->slof = %lu\n", lnb->slof);
	printf("\tfeparams.frequency = %d\n", feparams.frequency);
	printf("\tfeparams.inversion = %d\n", feparams.inversion);
	printf("\tfeparams.u.qpsk.symbol_rate = %d\n", feparams.u.qpsk.symbol_rate);
	printf("\tfeparams.u.qpsk.fec_inner = %d\n", feparams.u.qpsk.fec_inner);
#endif

	if (ioctl(adapter->fefd, FE_SET_FRONTEND, &feparams) < 0) {
		fprintf(stderr, "ioctl FE_SET_FRONTEND failed\n");
		exit(-1);
	}

	do {
		status=fe_get_event(adapter->fefd);
	} while (!(status && FE_HAS_LOCK) && !(status && FE_TIMEDOUT));

	/* TODO We should return the condition whether we suceeded */

	return 1;
}

